//  Copyright Rouven T. Kruse. All rights reserved.
//  Use of this source code is governed by an MIT-style license that can be
//  found in the LICENSE file.

import 'dart:convert';
import 'dart:io';

import 'package:dart_style/dart_style.dart';
import 'package:mustache/mustache.dart' as Mustache;
import 'package:path/path.dart' as Path;
import 'package:resource/resource.dart';

/// Base class for source file generators.
abstract class CageSourceGenerator {
  static final DartFormatter _dartFormatter = DartFormatter();

  final String classAppendix;

  final String fileNameAppendix;

  final String path;

  final bool removePathExtensions;

  String baseName;

  String className;

  String destinationPath;

  String dirName;

  String fileName;

  CageSourceGenerator(this.path,
      {this.classAppendix = '',
      this.fileNameAppendix = '',
      this.removePathExtensions = false}) {
    dirName = Path.dirname(path).replaceAll(RegExp('\/'), Path.separator);

    destinationPath = Path.join('lib', 'src', dirName != '.' ? dirName : '');

    baseName = Path.basename(path).toLowerCase().replaceAll(RegExp('-'), '_');

    fileName = baseName;

    if (removePathExtensions && RegExp('.*\.[a-zA-Z]}\$').hasMatch(fileName)) {
      fileName = fileName.replaceFirst(RegExp('.*(\.[a-zA-Z])\$'), '');
    }

    className =
        fileName[0].toUpperCase() + fileName.substring(1) + classAppendix;

    if (!fileName.endsWith(fileNameAppendix)) {
      fileName += fileNameAppendix;
    }

    fileName += '.dart';
  }

  /// Creates and renders a mustache template from given [path]. The template
  /// will receive the [fileName] passed and the passed [replacements].
  /// The [replacements] represent values that will replace variables in the
  /// template.
  Future<String> createTemplate(final String path, final String fileName,
      final dynamic replacements) async {
    final Resource resource = Resource('package:cage/$path');
    final String resourceString = await resource.readAsString(encoding: utf8);

    final Mustache.Template template =
        Mustache.Template(resourceString, name: fileName);

    return template.renderString(replacements);
  }

  /// Formats the [inputDartCode] with the [DartFormatter], so that all source
  /// files will be equally structured.
  String formatDartCode(final String inputDartCode) =>
      _dartFormatter.format(inputDartCode);

  Future<void> generate();

  /// Writes a file to the [path] with given [contents].
  ///
  /// If a file with that [path] already exists, a [StateError] is thrown.
  ///
  /// If any parent does not yet exist, it will be created.
  Future<void> writeFile(final String path, final String contents) async {
    final File file = File(path);

    if (file.existsSync()) {
      throw StateError('File $path already exists');
    }

    if (!file.parent.existsSync()) {
      file.parent.createSync(recursive: true);
    }

    file.writeAsStringSync(contents);

    print('Successfully generated file "$path"');
  }
}
